package com.ruoyi.jjh.declaration.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.jjh.declaration.entity.BmsTemplateInfo;
import com.ruoyi.jjh.declaration.mapper.BmsTemplateInfoMapper;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseDirectoryService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * 申报模板信息Service业务层处理
 *
 * @author farben
 * @date 2023-08-25
 */
@Service
public class BmsTemplateInfoServiceImpl extends ServiceImpl<BmsTemplateInfoMapper, BmsTemplateInfo> implements IBmsTemplateInfoService {

    @Autowired
    private IBmsEnterpriseDirectoryService enterpriseDirectoryService;

    /**
     * 查询申报模板信息列表
     *
     * @param bmsTemplateInfo 申报模板信息
     * @return 申报模板信息
     */
    @Override
    public Page<BmsTemplateInfo> selectBmsTemplateInfoList(Page<BmsTemplateInfo> page, BmsTemplateInfo bmsTemplateInfo) {
        return baseMapper.templateInfoList(page, bmsTemplateInfo);
    }

    /**
     * 新增项目时查询所有模板信息
     *
     * @param bmsTemplateInfo 申报模板信息
     * @return 申报模板信息集合
     */
    @Override
    public List<BmsTemplateInfo> selectBmsTemplateInfoList(BmsTemplateInfo bmsTemplateInfo) {
        return baseMapper.templateInfoList(bmsTemplateInfo);
    }

    /**
     * 导出模板表单
     *
     * @param id
     * @return
     * @author emiya.xie
     * @create 2023/8/31 10:25
     */
    @Override
    public void exportTemplate(HttpServletResponse response, Long id) {
        BmsTemplateInfo bmsTemplateInfo = baseMapper.selectById(id);
        JSONArray js1 = JSONObject.parse(bmsTemplateInfo.getTemplateJson()).getJSONArray("fields");
        List<List<String>> list = new ArrayList<>();
        for (Object o : js1) {
            JSONObject parse = JSONObject.parse(o.toString());
            List<String> head0 = new ArrayList<>();
            head0.add(parse.get("label").toString());
            list.add(head0);
        }
        String fileName = bmsTemplateInfo.getTemplateName() + ".xlsx";
        // 创建 Excel 写入流
        // 设置响应头，告诉浏览器下载文件
        // 设置响应头，告诉浏览器下载文件
        response.setContentType("application/vnd.ms-excel");
        response.setHeader("Content-Disposition", "attachment; filename="+fileName);
        try (ServletOutputStream outputStream = response.getOutputStream()) {
            // 使用 EasyExcel 写数据到响应流
            EasyExcel.write(outputStream)
                    // 这里放入动态头
                    .head(list).sheet("模板")
                    // 当然这里数据也可以用 List<List<String>> 去传入
                    .doWrite(new ArrayList<>());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }


    /**
     * 批量删除申报模板信息
     *
     * @param ids 需要删除的申报模板信息主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateInfoByIds(Long[] ids) {
        return baseMapper.deleteBmsTemplateInfoByIds(ids);
    }

    /**
     * 删除申报模板信息信息
     *
     * @param id 申报模板信息主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateInfoById(Long id) {
        return baseMapper.deleteBmsTemplateInfoById(id);
    }


    /**
     * 新增申报模板
     */
    @Override
    public Boolean add(BmsTemplateInfo bmsTemplateInfo) {
//        String str ="bms_"+ RandomUtil.randomString(5)+LocalDate.now().getMonthValue()+LocalDate.now().getDayOfMonth();
//        bmsTemplateInfo.setTableName(str);
        changeStatus(bmsTemplateInfo);
        //新增的时候根据当前时间判断状态是否有效
        return save(bmsTemplateInfo);
    }

    /**
     * 查询单个模板
     */
    @Override
    public BmsTemplateInfo selectByIdOne(Long id) {
        return baseMapper.selectByIdOne(id);
    }

    /**
     * 修改申报模板的状态
     */
    @Override
    public void changeStatus(BmsTemplateInfo items) {
        LocalDate currentDate = LocalDate.now();
        LocalDate startDate = LocalDate.of(items.getUseStart().getYear(), items.getUseStart().getMonth(), items.getUseStart().getDayOfMonth());
        LocalDate endDate = LocalDate.of(items.getUseEnd().getYear(), items.getUseEnd().getMonth(), items.getUseEnd().getDayOfMonth());
        if (currentDate.isBefore(startDate)) {
            items.setStatus(0);
        }
        if (currentDate.isAfter(endDate)) {
            items.setStatus(0);
        }
        if (currentDate.equals(startDate) || currentDate.equals(endDate)) {
            items.setStatus(1);
        }
        if (currentDate.isAfter(startDate) && currentDate.isBefore(endDate)) {
            items.setStatus(1);
        }
    }
}
