package com.ruoyi.jjh.ent.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.ent.entity.JPolicyFile;
import com.ruoyi.jjh.ent.entity.request.JPolicyFileRequest;
import com.ruoyi.jjh.ent.service.JMemorandumService;
import com.ruoyi.jjh.ent.service.JPolicyFileService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * 政策文件（j_policy_file）表控制层
 *
 * @author du
 * @since 2024/5/6 11:16
 */
@RestController
@RequestMapping("/jjh/jPolicyFile")
@Api(tags = "政策文件表")
public class JPolicyFileController extends BaseController {

    /**
     * 服务对象
     */
    @Resource
    private JPolicyFileService jPolicyFileService;

    @Resource
    private JMemorandumService jMemorandumService;

    /**
     * 分页查询所有数据
     *
     * @param page               分页对象
     * @param jPolicyFileRequest 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据", response = JPolicyFile.class)
    @GetMapping
    public AjaxResult selectAll(Page<JPolicyFile> page, JPolicyFileRequest jPolicyFileRequest) {
        return success(jPolicyFileService.page(page, jPolicyFileRequest));
    }

    @ApiOperation(value = "获取某个模板的已填报数据并且导出")
    @GetMapping("/getYtb")
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    public void getYtb(HttpServletResponse response, @RequestParam(name = "list") List<Long> list1) {
        List<BmsDeclarationRecords> ytb = jMemorandumService.getYtb(list1);
        int size1 = 0;
        if(ytb!=null&& !ytb.isEmpty()){
            Long trId = ytb.get(0).getTemplateRecordId();
            for (BmsDeclarationRecords item : ytb) {
                if (item.getTemplateRecordId().equals(trId)) {
                    size1++;
                }
            }
            if(size1!=ytb.size()){
                throw new ServiceException("请选择同一在线模板数据!");
            }
        }else {
            throw new ServiceException("请选择企业填报数据!");
        }

        List<List<String>> data = new ArrayList<>();

        List<List<String>> final1 = new ArrayList<>();
        // 创建一个 Map 用于存储不同的列表，键为字符串表示“变量名”，值为列表
        Map<String, List<String>> list = new LinkedHashMap<>();
        List<String> head0 = new ArrayList<>();
        head0.add("企业名称");
        List<String> head1 = new ArrayList<>();
        head1.add("统一社会信用代码");
        list.put("head0", head0);
        list.put("head1", head1);

        //根据数据添加近excel
        for (int i = 0; i < ytb.size(); i++) {
            JSONArray js1 = JSONObject.parse(ytb.get(i).getTemplateJson()).getJSONArray("fields");
            //要添加的数据
            List<String> addData = new ArrayList<>();
            addData.add(ytb.get(i).getMatter());
            addData.add(ytb.get(i).getCreditCode());
            for (int i1 = 0; i1 < js1.size(); i1++) {
                //获取每个组件
                JSONObject lb = JSON.parseObject(js1.get(i1).toString());
                if ("el-select".equals(lb.get("tag").toString()) || "el-radio-group".equals(lb.get("tag").toString())) {
                    List<String> head6 = new ArrayList<>();
                    head6.add(lb.get("label").toString());
                    list.put("headi" + i1, head6);
                    JSONArray op1 = JSONObject.parse(js1.get(i1).toString()).getJSONArray("options");
                        for (Object s1 : op1) {
                            JSONObject parse = JSONObject.parse(s1.toString());
                            if (parse.get("value").toString().equals(lb.get("defaultValue").toString())) {
                                addData.add(parse.get("label").toString());
                                break;
                            }
                        }
                } else  if ("el-checkbox-group".equals(lb.get("tag").toString())) {
                    List<String> head7 = new ArrayList<>();
                    head7.add(lb.get("label").toString());
                    list.put("headi" + i1, head7);
                    JSONArray op1 = JSONObject.parse(js1.get(i1).toString()).getJSONArray("options");
                    StringBuilder as = new StringBuilder();
                    JSONArray jsonArray = JSONObject.parse(js1.get(i1).toString()).getJSONArray("defaultValue");
                    for (Object o : jsonArray) {
                        for (Object s1 : op1) {
                            JSONObject parse = JSONObject.parse(s1.toString());
                            if(parse.get("value").toString().equals(o.toString())){
                                as.append(parse.get("label")).append(",");
                                break;
                            }
                        }
                    }
                    addData.add(as.toString());
                } else {
                    List<String> head2 = new ArrayList<>();
                    head2.add(lb.get("label").toString());
                    list.put("head2" + i1, head2);
                    addData.add(lb.get("defaultValue").toString());
                }
            }
            data.add(addData);
        }
        for (Map.Entry<String, List<String>> st : list.entrySet()) {
            final1.add(st.getValue());
        }

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-Disposition", "attachment; filename=data.xlsx");
        try  {
            ServletOutputStream outputStream = response.getOutputStream();
            // 使用 EasyExcel 写数据到响应流
            EasyExcel.write(outputStream)
                    // 这里放入动态头
                    .head(final1).sheet("模板")
                    // 当然这里数据也可以用 List<List<String>> 去传入
                    .doWrite(data);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据", response = JPolicyFile.class)
    @GetMapping("{id}")
    public AjaxResult selectOne(@PathVariable Serializable id) {
        return success(jPolicyFileService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param jPolicyFile 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    public AjaxResult insert(@RequestBody JPolicyFile jPolicyFile) {
        return success(jPolicyFileService.save(jPolicyFile));
    }

    /**
     * 修改数据
     *
     * @param jPolicyFile 实体对象
     * @return 修改结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "修改数据")
    @PostMapping("/edit")
    public AjaxResult update(@RequestBody JPolicyFile jPolicyFile) {
        return success(jPolicyFileService.updateById(jPolicyFile));
    }

    /**
     * 删除数据
     *
     * @param id 主键结合
     * @return 删除结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "删除数据")
    @PostMapping("/{id}")
    public AjaxResult delete(@PathVariable Long id) {
        return success(jPolicyFileService.removeById(id));
    }
}
